"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = fetchApi;
const lodash_1 = __importDefault(require("lodash"));
/**
 * # Fetch API
 */
function fetchApi(url, options, csrf, 
/**
 * Key to use to grab local Storage csrf value.
 */
localStorageCSRFKey, 
/**
 * Key with which to set the request header csrf
 * value
 */
csrfHeaderKey) {
    return __awaiter(this, void 0, void 0, function* () {
        let data;
        const csrfKey = "x-dsql-csrf-key";
        const csrfValue = localStorage.getItem(localStorageCSRFKey || csrfKey);
        let finalHeaders = {
            "Content-Type": "application/json",
        };
        if (csrf && csrfValue) {
            finalHeaders[localStorageCSRFKey || csrfKey] = csrfValue;
        }
        if (typeof options === "string") {
            try {
                let fetchData;
                switch (options) {
                    case "post":
                        fetchData = yield fetch(url, {
                            method: options,
                            headers: finalHeaders,
                        });
                        data = fetchData.json();
                        break;
                    default:
                        fetchData = yield fetch(url);
                        data = fetchData.json();
                        break;
                }
            }
            catch (error) {
                console.log("FetchAPI error #1:", error.message);
                data = null;
            }
        }
        else if (typeof options === "object") {
            try {
                let fetchData;
                if (options.body && typeof options.body === "object") {
                    let oldOptionsBody = lodash_1.default.cloneDeep(options.body);
                    options.body = JSON.stringify(oldOptionsBody);
                }
                if (options.headers) {
                    options.headers = lodash_1.default.merge(options.headers, finalHeaders);
                    const finalOptions = Object.assign({}, options);
                    fetchData = yield fetch(url, finalOptions);
                }
                else {
                    const finalOptions = Object.assign(Object.assign({}, options), { headers: finalHeaders });
                    fetchData = yield fetch(url, finalOptions);
                }
                data = fetchData.json();
            }
            catch (error) {
                console.log("FetchAPI error #2:", error.message);
                data = null;
            }
        }
        else {
            try {
                let fetchData = yield fetch(url);
                data = yield fetchData.json();
            }
            catch (error) {
                console.log("FetchAPI error #3:", error.message);
                data = null;
            }
        }
        return data;
    });
}
