"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = sendEmailCode;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const grab_host_names_1 = __importDefault(require("../../utils/grab-host-names"));
const api_send_email_code_1 = __importDefault(require("../../functions/api/users/api-send-email-code"));
/**
 * # Send Email Code to a User
 */
function sendEmailCode(params) {
    return __awaiter(this, void 0, void 0, function* () {
        const { key, email, database, temp_code_field_name, mail_domain, mail_password, mail_username, mail_port, sender, user_id, response, extraCookies, } = params;
        const grabedHostNames = (0, grab_host_names_1.default)();
        const { host, port, scheme } = grabedHostNames;
        const defaultTempLoginFieldName = "temp_login_code";
        const emailLoginTempCodeFieldName = temp_code_field_name
            ? temp_code_field_name
            : defaultTempLoginFieldName;
        const emailHtml = `<p>Please use this code to login</p>\n<h2>{{code}}</h2>\n<p>Please note that this code expires after 15 minutes</p>`;
        /**
         * Check for local DB settings
         *
         * @description Look for local db settings in `.env` file and by pass the http request if available
         */
        const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } = process.env;
        if ((DSQL_DB_HOST === null || DSQL_DB_HOST === void 0 ? void 0 : DSQL_DB_HOST.match(/./)) &&
            (DSQL_DB_USERNAME === null || DSQL_DB_USERNAME === void 0 ? void 0 : DSQL_DB_USERNAME.match(/./)) &&
            (DSQL_DB_PASSWORD === null || DSQL_DB_PASSWORD === void 0 ? void 0 : DSQL_DB_PASSWORD.match(/./)) &&
            (DSQL_DB_NAME === null || DSQL_DB_NAME === void 0 ? void 0 : DSQL_DB_NAME.match(/./)) &&
            global.DSQL_USE_LOCAL) {
            /** @type {import("../../types").DSQL_DatabaseSchemaType | undefined} */
            let dbSchema;
            try {
                const localDbSchemaPath = path_1.default.resolve(process.cwd(), "dsql.schema.json");
                dbSchema = JSON.parse(fs_1.default.readFileSync(localDbSchemaPath, "utf8"));
            }
            catch (error) { }
            return yield (0, api_send_email_code_1.default)({
                database: DSQL_DB_NAME,
                email,
                email_login_field: emailLoginTempCodeFieldName,
                html: emailHtml,
                mail_domain,
                mail_password,
                mail_port,
                mail_username,
                sender,
                response,
                extraCookies,
            });
        }
        else {
            /**
             * Make https request
             *
             * @description make a request to datasquirel.com
             *
             * @type {import("../../types").SendOneTimeCodeEmailResponse}
             */
            const httpResponse = yield new Promise((resolve, reject) => {
                const reqPayload = JSON.stringify({
                    email,
                    database,
                    email_login_field: emailLoginTempCodeFieldName,
                    mail_domain,
                    mail_password,
                    mail_username,
                    mail_port,
                    sender,
                    html: emailHtml,
                });
                const httpsRequest = scheme.request({
                    method: "POST",
                    headers: {
                        "Content-Type": "application/json",
                        "Content-Length": Buffer.from(reqPayload).length,
                        Authorization: key ||
                            process.env.DSQL_FULL_ACCESS_API_KEY ||
                            process.env.DSQL_API_KEY,
                    },
                    port,
                    hostname: host,
                    path: `/api/user/${user_id || grabedHostNames.user_id}/send-email-code`,
                }, 
                /**
                 * Callback Function
                 *
                 * @description https request callback
                 */
                (res) => {
                    var str = "";
                    res.on("data", function (chunk) {
                        str += chunk;
                    });
                    res.on("end", function () {
                        resolve(JSON.parse(str));
                    });
                    res.on("error", (err) => {
                        reject(err);
                    });
                });
                httpsRequest.write(reqPayload);
                httpsRequest.end();
            });
            return httpResponse;
        }
    });
}
