"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = queryDSQLAPI;
const path_1 = __importDefault(require("path"));
const grab_host_names_1 = __importDefault(require("../../utils/grab-host-names"));
const serialize_query_1 = __importDefault(require("../../utils/serialize-query"));
/**
 * # Query DSQL API
 */
function queryDSQLAPI(_a) {
    return __awaiter(this, arguments, void 0, function* ({ body, query, useDefault, path: passedPath, method, apiKey, }) {
        const grabedHostNames = (0, grab_host_names_1.default)({ useDefault });
        const { host, port, scheme } = grabedHostNames;
        try {
            /**
             * Make https request
             *
             * @description make a request to datasquirel.com
             */
            const httpResponse = yield new Promise((resolve, reject) => {
                const reqPayload = body ? JSON.stringify(body) : undefined;
                let headers = {
                    "Content-Type": "application/json",
                    Authorization: apiKey ||
                        (!method || method == "GET" || method == "get"
                            ? process.env.DSQL_READ_ONLY_API_KEY
                            : undefined) ||
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY,
                };
                if (reqPayload) {
                    headers["Content-Length"] = Buffer.from(reqPayload).length;
                }
                let finalPath = path_1.default.join("/", passedPath);
                if (query) {
                    const queryString = (0, serialize_query_1.default)(query);
                    finalPath += `${queryString}`;
                }
                const requestOptions = {
                    method: method || "GET",
                    headers,
                    port,
                    hostname: host,
                    path: finalPath,
                };
                const httpsRequest = scheme.request(requestOptions, 
                /**
                 * Callback Function
                 *
                 * @description https request callback
                 */
                (response) => {
                    var str = "";
                    response.on("data", function (chunk) {
                        str += chunk;
                    });
                    response.on("end", function () {
                        try {
                            resolve(JSON.parse(str));
                        }
                        catch (error) {
                            resolve({
                                success: false,
                                payload: undefined,
                                msg: `An error occurred while parsing the response`,
                                error: error.message,
                                errorData: { requestOptions, grabedHostNames },
                            });
                        }
                    });
                    response.on("error", (err) => {
                        resolve({
                            success: false,
                            payload: undefined,
                            msg: `An error occurred on the response`,
                            error: err.message,
                            errorData: { requestOptions, grabedHostNames },
                        });
                    });
                });
                httpsRequest.on("error", (err) => {
                    resolve({
                        success: false,
                        payload: undefined,
                        msg: `An error occurred while making the request`,
                        error: err.message,
                        errorData: { requestOptions, grabedHostNames },
                    });
                });
                if (reqPayload) {
                    httpsRequest.write(reqPayload);
                }
                httpsRequest.end();
            });
            return httpResponse;
        }
        catch (error) {
            return {
                success: false,
                payload: undefined,
                msg: `Request Failed`,
                error: error.message,
            };
        }
    });
}
