"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = addMariadbUser;
const generate_password_1 = __importDefault(require("generate-password"));
const DB_HANDLER_1 = __importDefault(require("../../utils/backend/global-db/DB_HANDLER"));
const NO_DB_HANDLER_1 = __importDefault(require("../../utils/backend/global-db/NO_DB_HANDLER"));
const addDbEntry_1 = __importDefault(require("./db/addDbEntry"));
const encrypt_1 = __importDefault(require("../dsql/encrypt"));
const LOCAL_DB_HANDLER_1 = __importDefault(require("../../utils/backend/global-db/LOCAL_DB_HANDLER"));
const grab_sql_key_name_1 = __importDefault(require("../../utils/grab-sql-key-name"));
/**
 * # Add Mariadb User
 */
function addMariadbUser(_a) {
    return __awaiter(this, arguments, void 0, function* ({ userId }) {
        try {
            const defaultMariadbUserHost = process.env.DSQL_DB_HOST || "127.0.0.1";
            const username = (0, grab_sql_key_name_1.default)({ type: "user", userId });
            const password = generate_password_1.default.generate({
                length: 16,
                numbers: true,
                symbols: true,
                uppercase: true,
                exclude: "*#.'`\"",
            });
            const encryptedPassword = (0, encrypt_1.default)({ data: password });
            const createMariadbUsersQuery = `CREATE USER IF NOT EXISTS '${username}'@'127.0.0.1' IDENTIFIED BY '${password}'`;
            if (global.DSQL_USE_LOCAL) {
                yield (0, LOCAL_DB_HANDLER_1.default)(createMariadbUsersQuery);
            }
            else {
                yield (0, NO_DB_HANDLER_1.default)(createMariadbUsersQuery);
            }
            const updateUserQuery = `UPDATE users SET mariadb_user = ?, mariadb_host = '127.0.0.1', mariadb_pass = ? WHERE id = ?`;
            const updateUserValues = [username, encryptedPassword, userId];
            const updateUser = global.DSQL_USE_LOCAL
                ? yield (0, LOCAL_DB_HANDLER_1.default)(updateUserQuery, updateUserValues)
                : yield (0, DB_HANDLER_1.default)(updateUserQuery, updateUserValues);
            const addMariadbUser = yield (0, addDbEntry_1.default)({
                tableName: "mariadb_users",
                data: {
                    user_id: userId,
                    username,
                    host: defaultMariadbUserHost,
                    password: encryptedPassword,
                    primary: "1",
                    grants: '[{"database":"*","table":"*","privileges":["ALL"]}]',
                },
                dbContext: "Master",
            });
            console.log(`User ${userId} SQL credentials successfully added.`);
        }
        catch ( /** @type {any} */error) {
            console.log(`Error in adding SQL user in 'addMariadbUser' function =>`, error.message);
        }
    });
}
////////////////////////////////////////////////
////////////////////////////////////////////////
////////////////////////////////////////////////
