"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = createDbSchemaFromDb;
const varDatabaseDbHandler_1 = __importDefault(require("../../functions/backend/varDatabaseDbHandler"));
const addDbEntry_1 = __importDefault(require("../../functions/backend/db/addDbEntry"));
const slugToCamelTitle_1 = __importDefault(require("../../shell/utils/slugToCamelTitle"));
const grab_dsql_schema_index_comment_1 = __importDefault(require("../../shell/utils/grab-dsql-schema-index-comment"));
const grab_required_database_schemas_1 = require("../../shell/createDbFromSchema/grab-required-database-schemas");
const numberfy_1 = __importDefault(require("../../utils/numberfy"));
const data_type_parser_1 = __importDefault(require("../../utils/db/schema/data-type-parser"));
const data_type_constructor_1 = __importDefault(require("../../utils/db/schema/data-type-constructor"));
function createDbSchemaFromDb(_a) {
    return __awaiter(this, arguments, void 0, function* ({ userId, database, dbId, }) {
        var _b, _c, _d, _e, _f, _g;
        try {
            if (!userId) {
                console.log("No user Id provided");
                return;
            }
            const targetDb = (0, grab_required_database_schemas_1.grabPrimaryRequiredDbSchema)({
                userId,
                dbId: database.db_schema_id || dbId,
            });
            if (!targetDb)
                throw new Error(`Target Db not found!`);
            const existingTables = yield (0, varDatabaseDbHandler_1.default)({
                database: database.db_full_name,
                queryString: `SHOW TABLES FROM ${database.db_full_name}`,
            });
            if (!existingTables)
                throw new Error("No Existing Tables");
            for (let i = 0; i < existingTables.length; i++) {
                const table = existingTables[i];
                const tableName = Object.values(table)[0];
                const tableInsert = yield (0, addDbEntry_1.default)({
                    dbFullName: "datasquirel",
                    tableName: "user_database_tables",
                    data: {
                        user_id: (0, numberfy_1.default)(userId),
                        db_id: database.id,
                        db_slug: database.db_slug,
                        table_name: (0, slugToCamelTitle_1.default)(tableName) || undefined,
                        table_slug: tableName,
                    },
                });
                const tableObject = {
                    tableName: tableName,
                    fields: [],
                    indexes: [],
                };
                const tableColumns = yield (0, varDatabaseDbHandler_1.default)({
                    database: database.db_full_name,
                    queryString: `SHOW COLUMNS FROM ${database.db_full_name}.${tableName}`,
                });
                if (tableColumns) {
                    for (let k = 0; k < tableColumns.length; k++) {
                        const tableColumn = tableColumns[k];
                        const { Field, Type, Null, Key, Default, Extra } = tableColumn;
                        const parsedDataType = (0, data_type_parser_1.default)(Type.toUpperCase());
                        const fieldObject = {
                            fieldName: Field,
                            dataType: (0, data_type_constructor_1.default)(parsedDataType.type, parsedDataType.limit, parsedDataType.decimal),
                        };
                        if (Null === null || Null === void 0 ? void 0 : Null.match(/^no$/i))
                            fieldObject.notNullValue = true;
                        if (Key === null || Key === void 0 ? void 0 : Key.match(/^pri$/i))
                            fieldObject.primaryKey = true;
                        if ((_b = Default === null || Default === void 0 ? void 0 : Default.toString()) === null || _b === void 0 ? void 0 : _b.match(/./))
                            fieldObject.defaultValue = Default;
                        if ((_c = Default === null || Default === void 0 ? void 0 : Default.toString()) === null || _c === void 0 ? void 0 : _c.match(/timestamp/i)) {
                            delete fieldObject.defaultValue;
                            fieldObject.defaultValueLiteral = Default;
                        }
                        if ((_d = Extra === null || Extra === void 0 ? void 0 : Extra.toString()) === null || _d === void 0 ? void 0 : _d.match(/auto_increment/i))
                            fieldObject.autoIncrement = true;
                        tableObject.fields.push(fieldObject);
                    }
                }
                const tableIndexes = yield (0, varDatabaseDbHandler_1.default)({
                    database: database.db_full_name,
                    queryString: `SHOW INDEXES FROM ${database.db_full_name}.${tableName}`,
                });
                if (tableIndexes) {
                    for (let m = 0; m < tableIndexes.length; m++) {
                        const indexObject = tableIndexes[m];
                        const { Table, Key_name, Column_name, Null, Index_type, Index_comment, } = indexObject;
                        if (!(Index_comment === null || Index_comment === void 0 ? void 0 : Index_comment.match(new RegExp((0, grab_dsql_schema_index_comment_1.default)()))))
                            continue;
                        const indexNewObject = {
                            indexType: (Index_type === null || Index_type === void 0 ? void 0 : Index_type.match(/fulltext/i))
                                ? "full_text"
                                : "regular",
                            indexName: Key_name,
                            indexTableFields: [],
                        };
                        const targetTableFieldObject = tableColumns === null || tableColumns === void 0 ? void 0 : tableColumns.filter((col) => col.Field === Column_name)[0];
                        const existingIndexField = (_e = tableObject.indexes) === null || _e === void 0 ? void 0 : _e.filter((indx) => indx.indexName == Key_name);
                        if (existingIndexField && existingIndexField[0]) {
                            (_f = existingIndexField[0].indexTableFields) === null || _f === void 0 ? void 0 : _f.push({
                                value: Column_name,
                                dataType: targetTableFieldObject.Type.toUpperCase(),
                            });
                        }
                        else {
                            indexNewObject.indexTableFields = [
                                {
                                    value: Column_name,
                                    dataType: targetTableFieldObject.Type.toUpperCase(),
                                },
                            ];
                            (_g = tableObject.indexes) === null || _g === void 0 ? void 0 : _g.push(indexNewObject);
                        }
                    }
                }
                targetDb.tables.push(tableObject);
            }
            (0, grab_required_database_schemas_1.writeUpdatedDbSchema)({ dbSchema: targetDb, userId });
            return true;
        }
        catch (error) {
            console.log(error);
            return false;
        }
    });
}
