"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = noDatabaseDbHandler;
const fs_1 = __importDefault(require("fs"));
const serverError_1 = __importDefault(require("./serverError"));
const NO_DB_HANDLER_1 = __importDefault(require("../../utils/backend/global-db/NO_DB_HANDLER"));
/**
 * # No Database DB Handler
 */
function noDatabaseDbHandler(queryString) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        ((_a = process.env.NODE_ENV) === null || _a === void 0 ? void 0 : _a.match(/dev/)) &&
            fs_1.default.appendFileSync("./.tmp/sqlQuery.sql", queryString + "\n" + Date() + "\n\n\n", "utf8");
        /**
         * Declare variables
         *
         * @description Declare "results" variable
         */
        let results;
        /**
         * Fetch from db
         *
         * @description Fetch data from db if no cache
         */
        try {
            /** ********************* Run Query */
            results = yield (0, NO_DB_HANDLER_1.default)(queryString);
            ////////////////////////////////////////
            ////////////////////////////////////////
            ////////////////////////////////////////
        }
        catch ( /** @type {any} */error) {
            (0, serverError_1.default)({
                component: "noDatabaseDbHandler",
                message: error.message,
            });
            console.log("ERROR in noDatabaseDbHandler =>", error.message);
        }
        /**
         * Return results
         *
         * @description Return results add to cache if "req" param is passed
         */
        if (results) {
            return results;
        }
        else {
            return null;
        }
    });
}
