"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = updateUsersTableSchema;
const serverError_1 = __importDefault(require("./serverError"));
const createDbFromSchema_1 = __importDefault(require("../../shell/createDbFromSchema"));
const grabSchemaFieldsFromData_1 = __importDefault(require("./grabSchemaFieldsFromData"));
const grab_required_database_schemas_1 = require("../../shell/createDbFromSchema/grab-required-database-schemas");
/**
 * # Add User Table to Database
 */
function updateUsersTableSchema(_a) {
    return __awaiter(this, arguments, void 0, function* ({ userId, database, newFields, newPayload, dbId, }) {
        var _b, _c;
        try {
            const dbFullName = database;
            let targetDatabase = (0, grab_required_database_schemas_1.grabPrimaryRequiredDbSchema)({
                dbId,
                userId,
            });
            if (!targetDatabase) {
                throw new Error("Couldn't Find Target Database!");
            }
            let existingTableIndex = targetDatabase === null || targetDatabase === void 0 ? void 0 : targetDatabase.tables.findIndex((table) => table.tableName === "users");
            const usersTable = targetDatabase.tables[existingTableIndex];
            if (!((_b = usersTable === null || usersTable === void 0 ? void 0 : usersTable.fields) === null || _b === void 0 ? void 0 : _b[0]))
                throw new Error("Users Table Not Found!");
            const additionalFields = (0, grabSchemaFieldsFromData_1.default)({
                fields: newFields,
                data: newPayload,
            });
            const spliceStartIndex = usersTable.fields.findIndex((field) => field.fieldName === "date_created");
            const finalSpliceStartIndex = spliceStartIndex >= 0 ? spliceStartIndex : 0;
            usersTable.fields.splice(finalSpliceStartIndex, 0, ...additionalFields);
            (0, grab_required_database_schemas_1.writeUpdatedDbSchema)({ dbSchema: targetDatabase, userId });
            const dbShellUpdate = yield (0, createDbFromSchema_1.default)({
                userId,
                targetDatabase: dbFullName,
            });
            return `Done!`;
        }
        catch (error) {
            (_c = global.ERROR_CALLBACK) === null || _c === void 0 ? void 0 : _c.call(global, `Error Updating User Table Schema`, error);
            (0, serverError_1.default)({
                component: "addUsersTableToDb",
                message: error.message,
                user: { id: userId },
            });
            return error.message;
        }
    });
}
