"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = handleMariadbUserGrantsForDatabasesRecreateGrants;
const types_1 = require("../../../types");
const grab_db_full_name_1 = __importDefault(require("../../../utils/grab-db-full-name"));
const dbHandler_1 = __importDefault(require("../../backend/dbHandler"));
const normalize_text_1 = __importDefault(require("../../../utils/normalize-text"));
function handleMariadbUserGrantsForDatabasesRecreateGrants(_a) {
    return __awaiter(this, arguments, void 0, function* ({ currentAccessedDatabase, user, updatedRecord, }) {
        const { accessedDatabase, dbSlug, allGrants, allTables, grants, tables } = currentAccessedDatabase;
        const dbFullName = (0, grab_db_full_name_1.default)({
            user,
            dbName: dbSlug,
        });
        if (allGrants && allTables) {
            const grantAllPrivileges = yield (0, dbHandler_1.default)({
                query: (0, normalize_text_1.default)(`
                    GRANT ALL PRIVILEGES ON \`${dbFullName}\`.* TO \
                    '${updatedRecord.username}'@'${updatedRecord.host}'
                `),
            });
            return { success: true };
        }
        if (allGrants && (tables === null || tables === void 0 ? void 0 : tables[0])) {
            for (let t = 0; t < tables.length; t++) {
                const table = tables[t];
                // queries.push(
                //     normalizeText(`
                //             GRANT ALL PRIVILEGES ON \`${dbFullName}\`.\`${table.tableSlug}\` \
                //             TO '${updatedRecord.username}'@'${updatedRecord.host}'
                //         `)
                // );
                const grantAllPrivilegesToTables = yield (0, dbHandler_1.default)({
                    query: (0, normalize_text_1.default)(`
                    GRANT ALL PRIVILEGES ON \`${dbFullName}\`.\`${table.tableSlug}\` \
                    TO '${updatedRecord.username}'@'${updatedRecord.host}'
                `),
                });
            }
            return { success: true };
        }
        if (grants === null || grants === void 0 ? void 0 : grants[0]) {
            const isGrantsInalid = grants.find((g) => !types_1.UserSQLPermissions.includes(g));
            if (isGrantsInalid) {
                return { msg: `grants is/are invalid!` };
            }
            if (tables === null || tables === void 0 ? void 0 : tables[0]) {
                for (let t = 0; t < tables.length; t++) {
                    const table = tables[t];
                    const grantSpecificPrivilegesToTables = yield (0, dbHandler_1.default)({
                        query: (0, normalize_text_1.default)(`
                        GRANT ${grants.join(",")} ON \
                        \`${dbFullName}\`.\`${table.tableSlug}\` TO \
                        '${updatedRecord.username}'@'${updatedRecord.host}'
                    `),
                    });
                }
                return { success: true };
            }
            else {
                const grantSecificPrivilegesToAllTables = yield (0, dbHandler_1.default)({
                    query: (0, normalize_text_1.default)(`
                    GRANT ${grants.join(",")} ON \
                    \`${dbFullName}\`.* TO \
                    '${updatedRecord.username}'@'${updatedRecord.host}'
                    `),
                });
                return { success: true };
            }
        }
        return { success: true };
    });
}
