"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = handleMariadbUserGrantsForDatabasesRecreateRecordsForDatabase;
const types_1 = require("../../../types");
const crud_1 = __importDefault(require("../../../utils/data-fetching/crud"));
const numberfy_1 = __importDefault(require("../../../utils/numberfy"));
function handleMariadbUserGrantsForDatabasesRecreateRecordsForDatabase(_a) {
    return __awaiter(this, arguments, void 0, function* ({ currentAccessedDatabase, user, updatedRecord, }) {
        const { accessedDatabase, dbSlug, allGrants, allTables, grants, tables } = currentAccessedDatabase;
        const insertSQLDbRecord = yield (0, crud_1.default)({
            action: "insert",
            table: "mariadb_user_databases",
            data: {
                all_privileges: allGrants ? 1 : 0,
                all_tables: allTables ? 1 : 0,
                db_id: (0, numberfy_1.default)(accessedDatabase.dbId),
                db_slug: accessedDatabase.dbSlug,
                db_schema_id: (0, numberfy_1.default)(accessedDatabase.dbSchemaId),
                user_id: user.id,
                mariadb_user_id: updatedRecord.id,
            },
        });
        if (tables === null || tables === void 0 ? void 0 : tables[0]) {
            for (let t = 0; t < tables.length; t++) {
                const table = tables[t];
                const insertTable = yield (0, crud_1.default)({
                    action: "insert",
                    table: "mariadb_user_tables",
                    data: {
                        all_privileges: allGrants ? 1 : 0,
                        all_fields: 1,
                        user_id: user.id,
                        mariadb_user_id: updatedRecord.id,
                        table_slug: table.tableSlug,
                        db_id: (0, numberfy_1.default)(table.dbId),
                        db_slug: table.dbSlug,
                        db_schema_id: (0, numberfy_1.default)(table.dbSchemaId),
                    },
                });
            }
        }
        if (grants === null || grants === void 0 ? void 0 : grants[0]) {
            const isGrantsInalid = grants.find((g) => !types_1.UserSQLPermissions.includes(g));
            if (isGrantsInalid) {
                return { msg: `grants is/are invalid!` };
            }
            for (let t = 0; t < grants.length; t++) {
                const grant = grants[t];
                yield (0, crud_1.default)({
                    action: "insert",
                    table: "mariadb_user_privileges",
                    data: {
                        user_id: user.id,
                        mariadb_user_id: updatedRecord.id,
                        privilege: grant,
                        db_id: (0, numberfy_1.default)(accessedDatabase.dbId),
                        db_slug: accessedDatabase.dbSlug,
                        db_schema_id: (0, numberfy_1.default)(accessedDatabase.dbSchemaId),
                    },
                });
            }
        }
        return { success: true };
    });
}
