"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = handleIndexescreateDbFromSchema;
const varDatabaseDbHandler_1 = __importDefault(require("../utils/varDatabaseDbHandler"));
const grab_dsql_schema_index_comment_1 = __importDefault(require("../utils/grab-dsql-schema-index-comment"));
/**
 * Handle DATASQUIREL Table Indexes
 * ===================================================
 * @description Iterate through each datasquirel schema
 * table index(if available), and perform operations
 */
function handleIndexescreateDbFromSchema(_a) {
    return __awaiter(this, arguments, void 0, function* ({ dbFullName, tableName, indexes, }) {
        const allExistingIndexes = yield (0, varDatabaseDbHandler_1.default)({
            queryString: `SHOW INDEXES FROM \`${dbFullName}\`.\`${tableName}\``,
        });
        for (let g = 0; g < indexes.length; g++) {
            const { indexType, indexName, indexTableFields, alias } = indexes[g];
            if (!(alias === null || alias === void 0 ? void 0 : alias.match(/./)))
                continue;
            /**
             * @description Check for existing Index in MYSQL db
             */
            try {
                const existingKeyInDb = allExistingIndexes.filter((indexObject) => indexObject.Key_name === alias);
                if (!existingKeyInDb[0])
                    throw new Error("This Index Does not Exist");
            }
            catch (error) {
                /**
                 * @description Create new index if determined that it
                 * doesn't exist in MYSQL db
                 */
                const queryString = `CREATE${indexType == "full_text" ? " FULLTEXT" : ""} INDEX \`${alias}\` ON \`${dbFullName}\`.\`${tableName}\`(${indexTableFields === null || indexTableFields === void 0 ? void 0 : indexTableFields.map((nm) => nm.value).map((nm) => `\`${nm}\``).join(",")}) COMMENT '${(0, grab_dsql_schema_index_comment_1.default)()} ${indexName}'`;
                const addIndex = yield (0, varDatabaseDbHandler_1.default)({ queryString });
            }
        }
        const allExistingIndexesAfterUpdate = yield (0, varDatabaseDbHandler_1.default)({
            queryString: `SHOW INDEXES FROM \`${dbFullName}\`.\`${tableName}\``,
        });
    });
}
