"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("dotenv").config({ path: "../../.env" });
const generate_password_1 = __importDefault(require("generate-password"));
const noDatabaseDbHandler_1 = __importDefault(require("../utils/noDatabaseDbHandler"));
const dbHandler_1 = __importDefault(require("../../functions/backend/dbHandler"));
const encrypt_1 = __importDefault(require("../../functions/dsql/encrypt"));
const grab_sql_key_name_1 = __importDefault(require("../../utils/grab-sql-key-name"));
/**
 * # Reset SQL Passwords
 */
function resetSQLCredentialsPasswords() {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        const users = (yield (0, dbHandler_1.default)({
            query: `SELECT * FROM users`,
        }));
        if (!users) {
            process.exit();
        }
        for (let i = 0; i < users.length; i++) {
            const user = users[i];
            if (!user)
                continue;
            try {
                const maridbUsers = (yield (0, dbHandler_1.default)({
                    query: `SELECT * FROM mysql.user WHERE User = '${(0, grab_sql_key_name_1.default)({ type: "user", userId: user.id })}'`,
                }));
                for (let j = 0; j < maridbUsers.length; j++) {
                    const { User, Host } = maridbUsers[j];
                    const password = generate_password_1.default.generate({
                        length: 16,
                        numbers: true,
                        symbols: true,
                        uppercase: true,
                        exclude: "*#.'`\"",
                    });
                    const encryptedPassword = (0, encrypt_1.default)({
                        data: password,
                        encryptionKey: process.env.DSQL_ENCRYPTION_PASSWORD,
                        encryptionSalt: process.env.DSQL_ENCRYPTION_SALT,
                    });
                    yield (0, noDatabaseDbHandler_1.default)(`SET PASSWORD FOR '${User}'@'${Host}' = PASSWORD('${password}')`);
                    if (user.mariadb_user == User && user.mariadb_host == Host) {
                        const updateUser = yield (0, dbHandler_1.default)({
                            query: `UPDATE users SET mariadb_pass = ? WHERE id = ?`,
                            values: [encryptedPassword, user.id],
                        });
                    }
                    console.log(`User ${user.id}: ${user.first_name} ${user.last_name} Password Updated successfully added.`);
                }
            }
            catch (error) {
                (_a = global.ERROR_CALLBACK) === null || _a === void 0 ? void 0 : _a.call(global, `Error Resetting MariaDB User Password`, error);
            }
        }
        process.exit();
    });
}
resetSQLCredentialsPasswords();
