"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = dsqlCrud;
const sql_delete_generator_1 = __importDefault(require("../../functions/dsql/sql/sql-delete-generator"));
const crud_get_1 = __importDefault(require("./crud-get"));
const conn_db_handler_1 = __importDefault(require("../db/conn-db-handler"));
const addDbEntry_1 = __importDefault(require("../../functions/backend/db/addDbEntry"));
const updateDbEntry_1 = __importDefault(require("../../functions/backend/db/updateDbEntry"));
function dsqlCrud(params) {
    return __awaiter(this, void 0, void 0, function* () {
        const { action, data, table, targetValue, sanitize, targetField, targetId, dbFullName, deleteData, batchData, deleteKeyValues, } = params;
        const finalData = (sanitize ? sanitize({ data }) : data);
        const finalBatchData = (sanitize ? sanitize({ batchData }) : batchData);
        const DB_CONN = global.DSQL_READ_ONLY_DB_CONN || global.DSQL_DB_CONN;
        switch (action) {
            case "get":
                return yield (0, crud_get_1.default)(params);
            // case "batch-get":
            //     return await dsqlCrudBatchGet(params);
            case "insert":
                const INSERT_RESULT = yield (0, addDbEntry_1.default)({
                    data: finalData,
                    batchData: finalBatchData,
                    tableName: table,
                    dbFullName,
                });
                return INSERT_RESULT;
            case "update":
                data === null || data === void 0 ? true : delete data.id;
                const UPDATE_RESULT = yield (0, updateDbEntry_1.default)({
                    data: finalData,
                    tableName: table,
                    dbFullName,
                    identifierColumnName: (targetField || "id"),
                    identifierValue: String(targetValue || targetId),
                });
                return UPDATE_RESULT;
            case "delete":
                const deleteQuery = (0, sql_delete_generator_1.default)({
                    data: targetId
                        ? { id: targetId }
                        : targetField && targetValue
                            ? { [targetField]: targetValue }
                            : deleteData,
                    tableName: table,
                    dbFullName,
                    deleteKeyValues,
                });
                const res = (yield (0, conn_db_handler_1.default)(DB_CONN, deleteQuery === null || deleteQuery === void 0 ? void 0 : deleteQuery.query, deleteQuery === null || deleteQuery === void 0 ? void 0 : deleteQuery.values));
                return {
                    success: Boolean(res.affectedRows),
                    payload: res,
                    queryObject: {
                        sql: (deleteQuery === null || deleteQuery === void 0 ? void 0 : deleteQuery.query) || "",
                        params: (deleteQuery === null || deleteQuery === void 0 ? void 0 : deleteQuery.values) || [],
                    },
                };
            default:
                return {
                    success: false,
                    payload: undefined,
                    msg: "Invalid action",
                };
        }
    });
}
