require("dotenv").config({ path: "../../.env" });
import generator from "generate-password";
import noDatabaseDbHandler from "../utils/noDatabaseDbHandler";
import dbHandler from "../../functions/backend/dbHandler";
import encrypt from "../../functions/dsql/encrypt";
import grabSQLKeyName from "../../utils/grab-sql-key-name";

/**
 * # Reset SQL Passwords
 */
async function resetSQLCredentialsPasswords() {
    const users = (await dbHandler({
        query: `SELECT * FROM users`,
    })) as any[];

    if (!users) {
        process.exit();
    }

    for (let i = 0; i < users.length; i++) {
        const user = users[i];

        if (!user) continue;

        try {
            const maridbUsers = (await dbHandler({
                query: `SELECT * FROM mysql.user WHERE User = '${grabSQLKeyName(
                    { type: "user", userId: user.id }
                )}'`,
            })) as any[];

            for (let j = 0; j < maridbUsers.length; j++) {
                const { User, Host } = maridbUsers[j];

                const password = generator.generate({
                    length: 16,
                    numbers: true,
                    symbols: true,
                    uppercase: true,
                    exclude: "*#.'`\"",
                });

                const encryptedPassword = encrypt({
                    data: password,
                    encryptionKey: process.env.DSQL_ENCRYPTION_PASSWORD,
                    encryptionSalt: process.env.DSQL_ENCRYPTION_SALT,
                });

                await noDatabaseDbHandler(
                    `SET PASSWORD FOR '${User}'@'${Host}' = PASSWORD('${password}')`
                );

                if (user.mariadb_user == User && user.mariadb_host == Host) {
                    const updateUser = await dbHandler({
                        query: `UPDATE users SET mariadb_pass = ? WHERE id = ?`,
                        values: [encryptedPassword, user.id],
                    });
                }

                console.log(
                    `User ${user.id}: ${user.first_name} ${user.last_name} Password Updated successfully added.`
                );
            }
        } catch (error: any) {
            global.ERROR_CALLBACK?.(
                `Error Resetting MariaDB User Password`,
                error as Error
            );
        }
    }

    process.exit();
}

resetSQLCredentialsPasswords();
