import sqlGenerator from "../../functions/dsql/sql/sql-generator";
import { APIResponseObject, DsqlCrudParam } from "../../types";
import connDbHandler, { ConnDBHandlerQueryObject } from "../db/conn-db-handler";

export default async function <
    T extends { [key: string]: any } = { [key: string]: any }
>({
    table,
    query,
    count,
    countOnly,
    dbFullName,
}: Omit<
    DsqlCrudParam<T>,
    "action" | "data" | "sanitize"
>): Promise<APIResponseObject> {
    let queryObject: ReturnType<Awaited<typeof sqlGenerator>> | undefined;

    queryObject = sqlGenerator({
        tableName: table,
        genObject: query,
        dbFullName,
    });

    const DB_CONN = global.DSQL_READ_ONLY_DB_CONN || global.DSQL_DB_CONN;

    let connQueries: ConnDBHandlerQueryObject[] = [
        {
            query: queryObject?.string,
            values: queryObject?.values || [],
        },
    ];

    const countQueryObject =
        count || countOnly
            ? sqlGenerator({
                  tableName: table,
                  genObject: query,
                  count: true,
                  dbFullName,
              })
            : undefined;

    if (count && countQueryObject) {
        connQueries.push({
            query: countQueryObject.string,
            values: countQueryObject.values,
        });
    } else if (countOnly && countQueryObject) {
        connQueries = [
            {
                query: countQueryObject.string,
                values: countQueryObject.values,
            },
        ];
    }

    const res = await connDbHandler(DB_CONN, connQueries);

    const isSuccess = Array.isArray(res) && Array.isArray(res[0]);

    return {
        success: isSuccess,
        payload: isSuccess ? (countOnly ? null : res[0]) : null,
        batchPayload: isSuccess ? (countOnly ? null : res) : null,
        error: isSuccess ? undefined : res?.error,
        errors: res?.errors,
        queryObject: {
            sql: queryObject?.string,
            params: queryObject?.values,
        },
        count: isSuccess
            ? res[1]?.[0]?.["COUNT(*)"]
                ? res[1][0]["COUNT(*)"]
                : res[0]?.[0]?.["COUNT(*)"]
                ? res[0][0]["COUNT(*)"]
                : undefined
            : undefined,
    };
}
